[![Gem Version](https://badge.fury.io/rb/schema_plus_triggers.svg)](http://badge.fury.io/rb/schema_plus_triggers)
[![Build Status](https://secure.travis-ci.org/SchemaPlus/schema_plus_triggers.svg)](http://travis-ci.org/SchemaPlus/schema_plus_triggers)
[![Coverage Status](https://img.shields.io/coveralls/SchemaPlus/schema_plus_triggers.svg)](https://coveralls.io/r/SchemaPlus/schema_plus_triggers)
[![Dependency Status](https://gemnasium.com/SchemaPlus/schema_plus_triggers.svg)](https://gemnasium.com/SchemaPlus/schema_plus_triggers)

# SchemaPlus::Triggers

ScemaPlus::Triggers adds support for SQL triggers in ActiveRecord.

SchemaPlus::Triggers is part of the [SchemaPlus](https://github.com/SchemaPlus/) family of Ruby on Rails ActiveRecord extension gems.

## Installation

<!-- SCHEMA_DEV: TEMPLATE INSTALLATION - begin -->
<!-- These lines are auto-inserted from a schema_dev template -->
As usual:

```ruby
gem "schema_plus_triggers"                # in a Gemfile
gem.add_dependency "schema_plus_triggers" # in a .gemspec
```

<!-- SCHEMA_DEV: TEMPLATE INSTALLATION - end -->

## Compatibility

SchemaPlus::Triggers is tested on:

<!-- SCHEMA_DEV: MATRIX - begin -->
<!-- These lines are auto-generated by schema_dev based on schema_dev.yml -->
* ruby **2.3.1** with activerecord **5.2**, using **postgresql**

<!-- SCHEMA_DEV: MATRIX - end -->

## Usage

### Note for PostgreSQL

You will need to also add the schema_plus_functions gem and define your trigger function.

    create_function :my_trigger_func, '', <<-END
    RETURNS trigger
      LANGUAGE plpgsql
      AS $$
    BEGIN
      BEGIN
        IF (TG_OP = 'DELETE') THEN
          INSERT INTO log (operation, log) VALUES ('DELETE', OLD.name);
        ELSEIF (TG_OP = 'UPDATE') THEN
          INSERT INTO log (operation, log) VALUES ('UPDATE', OLD.name || ' to ' || NEW.name);
        ELSEIF (TG_OP = 'INSERT') THEN
          INSERT INTO log (operation, log) VALUES ('INSERT', NEW.name);
        END IF; 
      END;
    END;
    $$
    END

### Migrations

To declare a trigger use `create_trigger`:

    create_trigger :my_table, :my_trigger, 'after insert', 'for each row execute procedure my_trigger_func()' 

To drop a trigger use `drop_trigger`:

    drop_trigger :my_table, :my_trigger

### Introspection

You can query the list of triggers defined at the connection level (uncached):

    connection.triggers

This will return an array of arrays. The inner array containing the table name and the trigger name.

## History

* **0.2.0** - Add testing for multiple PostgreSQL versions
* **0.1.0** - Initial release

## Development & Testing

Are you interested in contributing to SchemaPlus::Triggers?  Thanks!  Please follow
the standard protocol: fork, feature branch, develop, push, and issue pull
request.

Some things to know about to help you develop and test:

<!-- SCHEMA_DEV: TEMPLATE USES SCHEMA_DEV - begin -->
<!-- These lines are auto-inserted from a schema_dev template -->
* **schema_dev**:  SchemaPlus::Triggers uses [schema_dev](https://github.com/SchemaPlus/schema_dev) to
  facilitate running rspec tests on the matrix of ruby, activerecord, and database
  versions that the gem supports, both locally and on
  [travis-ci](http://travis-ci.org/SchemaPlus/schema_plus_triggers)

  To to run rspec locally on the full matrix, do:

        $ schema_dev bundle install
        $ schema_dev rspec

  You can also run on just one configuration at a time;  For info, see `schema_dev --help` or the [schema_dev](https://github.com/SchemaPlus/schema_dev) README.

  The matrix of configurations is specified in `schema_dev.yml` in
  the project root.


<!-- SCHEMA_DEV: TEMPLATE USES SCHEMA_DEV - end -->

<!-- SCHEMA_DEV: TEMPLATE USES SCHEMA_PLUS_CORE - begin -->
<!-- These lines are auto-inserted from a schema_dev template -->
* **schema_plus_core**: SchemaPlus::Triggers uses the SchemaPlus::Core API that
  provides middleware callback stacks to make it easy to extend
  ActiveRecord's behavior.  If that API is missing something you need for
  your contribution, please head over to
  [schema_plus_core](https://github.com/SchemaPlus/schema_plus_core) and open
  an issue or pull request.

<!-- SCHEMA_DEV: TEMPLATE USES SCHEMA_PLUS_CORE - end -->

<!-- SCHEMA_DEV: TEMPLATE USES SCHEMA_MONKEY - begin -->
<!-- These lines are auto-inserted from a schema_dev template -->
* **schema_monkey**: SchemaPlus::Triggers is implemented as a
  [schema_monkey](https://github.com/SchemaPlus/schema_monkey) client,
  using [schema_monkey](https://github.com/SchemaPlus/schema_monkey)'s
  convention-based protocols for extending ActiveRecord and using middleware stacks.

<!-- SCHEMA_DEV: TEMPLATE USES SCHEMA_MONKEY - end -->
